#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2010 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows.Forms;
using HeuristicLab.MainForm;
using HeuristicLab.MainForm.WindowsForms;

namespace HeuristicLab.Clients.OKB {
  [View("DataType View")]
  [Content(typeof(DataType), true)]
  public partial class DataTypeView : OKBItemView {
    private List<Platform> platformComboBoxValues;

    public new DataType Content {
      get { return (DataType)base.Content; }
      set { base.Content = value; }
    }

    public DataTypeView() {
      InitializeComponent();
    }

    protected override void OnInitialized(System.EventArgs e) {
      base.OnInitialized(e);
      platformComboBoxValues = OKBClient.Instance.Platforms.ToList();
      platformComboBox.DataSource = platformComboBoxValues;
    }

    protected override void OnContentChanged() {
      base.OnContentChanged();
      if (Content == null) {
        nameTextBox.Text = string.Empty;
        sqlNameComboBox.SelectedIndex = -1;
        platformComboBox.SelectedIndex = -1;
        if (ViewAttribute.HasViewAttribute(this.GetType()))
          this.Caption = ViewAttribute.GetViewName(this.GetType());
        else
          this.Caption = "DataType View";
      } else {
        nameTextBox.Text = Content.Name;
        sqlNameComboBox.Text = Content.SqlName;
        platformComboBox.SelectedItem = platformComboBoxValues.FirstOrDefault(p => p.Id == Content.PlatformId);
        Caption = Content.Name;
      }
    }

    protected override void SetEnabledStateOfControls() {
      base.SetEnabledStateOfControls();
      nameTextBox.Enabled = Content != null;
      nameTextBox.ReadOnly = ReadOnly;
      sqlNameComboBox.Enabled = (Content != null) && !ReadOnly;
      platformComboBox.Enabled = (Content != null) && !ReadOnly;
    }

    protected override void OnContentPropertyChanged(string propertyName) {
      switch (propertyName) {
        case "Name":
          nameTextBox.Text = Content.Name;
          Caption = Content.Name;
          break;
        case "SqlName":
          sqlNameComboBox.Text = Content.SqlName;
          break;
        case "PlatformId":
          platformComboBox.SelectedItem = platformComboBoxValues.FirstOrDefault(p => p.Id == Content.PlatformId);
          break;
      }
    }

    private void nameTextBox_KeyDown(object sender, KeyEventArgs e) {
      if ((e.KeyCode == Keys.Enter) || (e.KeyCode == Keys.Return))
        nameLabel.Focus();  // set focus on label to validate data
      if (e.KeyCode == Keys.Escape) {
        nameTextBox.Text = Content.Name;
        nameLabel.Focus();  // set focus on label to validate data
      }
    }
    private void nameTextBox_TextChanged(object sender, EventArgs e) {
      if (nameTextBox.Text != Content.Name)
        Content.Name = nameTextBox.Text;
    }
    private void sqlNameComboBox_SelectedValueChanged(object sender, EventArgs e) {
      if (sqlNameComboBox.Text != Content.SqlName) {
        Content.SqlName = sqlNameComboBox.Text;
      }
    }
    private void platformComboBox_SelectedValueChanged(object sender, System.EventArgs e) {
      if (Content != null) {
        Platform selected = platformComboBox.SelectedItem as Platform;
        if (selected != null) Content.PlatformId = selected.Id;
      }
    }
  }
}
